@@ -1,100 +1,6 @@
-#' Simulated example data with continuous outcome and quadratic covariate effects
-#'
-#' A dataset containing simulated data where the outcome depends quadratically
-#' on a partially observed covariate.
-#'
-#' @format A data frame with 1000 rows and 5 variables:
-#' \describe{
-#'   \item{y}{Continuous outcome}
-#'   \item{z}{Fully observed covariate, with linear effect on outcome}
-#'   \item{x}{Partially observed normally distributed covariate, with quadratic effect on outcome}
-#'   \item{xsq}{The square of x, which thus has missing values also}
-#'   \item{v}{An auxiliary variable (i.e. not contained in the substantive model)}
-#' }
-#'
-"ex_linquad"
-
-#' Simulated example data with continuous outcome and interaction between two partially observed covariates
-#'
-#' A dataset containing simulated data where the outcome depends on both main
-#' effects and interaction of two partially observed covariates.
-#'
-#' @format A data frame with 1000 rows and 4 variables:
-#' \describe{
-#'   \item{y}{Continuous outcome}
-#'   \item{x1}{Partially observed normally distributed covariate}
-#'   \item{x2}{Partially obserevd binary covariate}
-#' }
-#'
-"ex_lininter"
-
-#' Simulated example data with binary outcome and quadratic covariate effects
-#'
-#' A dataset containing simulated data where the binary outcome depends quadratically
-#' on a partially observed covariate.
-#'
-#' @format A data frame with 1000 rows and 5 variables:
-#' \describe{
-#'   \item{y}{Binary outcome}
-#'   \item{z}{Fully observed covariate, with linear effect on outcome (on log odds scale)}
-#'   \item{x}{Partially observed normally distributed covariate, with quadratic effect on outcome (on log odds scale)}
-#'   \item{xsq}{The square of x, which thus has missing values also}
-#'   \item{v}{An auxiliary variable (i.e. not contained in the substantive model)}
-#' }
-#'
-"ex_logisticquad"
-
-#' Simulated example data with count outcome, modelled using Poisson regression
-#'
-#' A dataset containing simulated data where the count outcome depends on two
-#' covariates, x and z, with missing values in x. The substantive model is
-#' Poisson regression.
-#'
-#' @format A data frame with 1000 rows and 3 variables:
-#' \describe{
-#'   \item{y}{Count outcome}
-#'   \item{z}{Fully observed covariate, with linear effect on outcome}
-#'   \item{x}{Partially observed normally distributed covariate, with linear effect on outcome}
-#' }
-#'
-"ex_poisson"
-
-#' Simulated example data with time to event outcome and quadratic covariate effects
-#'
-#' A dataset containing simulated data where a time to event outcome depends quadratically
-#' on a partially observed covariate.
-#'
-#' @format A data frame with 1000 rows and 6 variables:
-#' \describe{
-#'   \item{t}{Time to event or censoring}
-#'   \item{d}{Binary indicator of whether event occurred or individual was censored}
-#'   \item{z}{Fully observed covariate, with linear effect on outcome (on log hazard scale)}
-#'   \item{x}{Partially observed normally distributed covariate, with quadratic effect on outcome (on log hazard scale)}
-#'   \item{xsq}{The square of x, which thus has missing values also}
-#'   \item{v}{An auxiliary variable (i.e. not contained in the substantive model)}
-#' }
-#'
-"ex_coxquad"
-
-#' Simulated example data with competing risks outcome and partially observed covariates
-#'
-#' A dataset containing simulated competing risks data. There are two competing risks, and
-#' some times are also censored.
-#'
-#' @format A data frame with 1000 rows and 4 variables:
-#' \describe{
-#'   \item{t}{Time to event or censoring}
-#'   \item{d}{Indicator of whether event 1 occurred (d=1), event 2 occured (d=2) or individual was censored (d=0)}
-#'   \item{x1}{Partially observed binary covariate, with linear effects on log competing risk hazards}
-#'   \item{x2}{Partially observed normally distributed (conditional on x1) covariate, with linear effects
-#'   on log competing risk hazards}
-#' }
-#'
-"ex_compet"
-
 #' Substantive model compatible fully conditional specification imputation of covariates.
 #'
-#' Multiple imputes missing covariate values using substantive model compatible
+#' Multiply imputes missing covariate values using substantive model compatible
 #' fully conditional specification.
 #'
 #' smcfcs imputes missing values of covariates using the Substantive Model Compatible
@@ -190,6 +96,68 @@
 
 #' @export
 smcfcs <- function(originaldata,smtype,smformula,method,predictorMatrix=NULL,m=5,numit=10,rjlimit=1000,noisy=FALSE) {
+  #call core  smcfcs function, passing through arguments
+  smcfcs.core(originaldata,smtype,smformula,method,predictorMatrix,m,numit,rjlimit,noisy)
+}
+
+#' Substantive model compatible fully conditional specification imputation of covariates for case cohort studies
+#'
+#' Multiply imputes missing covariate values using substantive model compatible
+#' fully conditional specification for case cohort studies.
+#'
+#' This version of \code{smcfcs} is designed for use with case cohort studies but where the analyst does not wish to,
+#' or cannot (due to not having the necessary data) impute the full cohort. The function's arguments are the same
+#' as for the main smcfcs function, except for \code{smformula}, \code{in.subco}, and \code{sampfrac} - see above
+#' for details on how these should be specified.
+#'
+#' @author Ruth Keogh \email{ruth.keogh@@lshtm.ac.uk}
+#' @author Jonathan Bartlett \email{jwb133@@googlemail.com}
+#'
+#' @param originaldata The case-cohort data set (NOT a full cohort data set with a case-cohort substudy within it)
+#' @param smformula A formula of the form "Surv(entertime,t,d)~x", where d is the event (d=1) or censoring (d=0) indicator, t is the event or censoring time and entertime is equal to the time origin (typically 0) for individuals in the subcohort and is equal to (t-0.001) for cases outside the subcohort [this sets cases outside the subcohort to enter follow-up just before their event time. The value 0.001 may need to be modified depending on the time scale.]
+#' @param in.subco The name of a column in the dataset with 0/1s that indicates whether the subject is in the subcohort
+#' @param sampfrac The proportion of individuals from the underlying full cohort who are in the subcohort
+#'
+#' @inheritParams smcfcs
+#'
+#' @example data-raw/cc_example.r
+#'
+#' @export
+smcfcs.casecohort <- function(originaldata,smformula,sampfrac,in.subco,method,predictorMatrix=NULL,m=5,numit=10,rjlimit=1000,noisy=FALSE) {
+  smcfcs.core(originaldata,smtype="casecohort",smformula,method,predictorMatrix,m,numit,rjlimit,noisy,sampfrac=sampfrac,in.subco=in.subco)
+}
+
+#' Substantive model compatible fully conditional specification imputation of covariates for nested case control
+#' studies
+#'
+#' Multiply imputes missing covariate values using substantive model compatible
+#' fully conditional specification for nested case control studies.
+#'
+#' This version of \code{smcfcs} is designed for use with nested case control studies. The function's arguments are the same
+#' as for the main smcfcs function, except for \code{smformula}, \code{set}, \code{event} and \code{nrisk} - see above
+#' for details on how these should be specified.
+#'
+#' @author Ruth Keogh \email{ruth.keogh@@lshtm.ac.uk}
+#' @author Jonathan Bartlett \email{jwb133@@googlemail.com}
+#'
+#' @param originaldata The nested case-control data set (NOT a full cohort data set with a case-cohort substudy within it)
+#' @param smformula A formula of the form "Surv(t,case)~x+strata(set)", where case is case-control indicator, t is the event or censoring time. Note that t could be set to the case's event time for the matched controls in a given set. The right hand side should include the case control set as a strata term (see example).
+#' @param set variable identifying matched sets in nested case-control study
+#' @param event variable which indicates who is a case/control in the nested case-control sample. Note that this is distinct from d.
+#' @param nrisk variable which is the number at risk (in the underlying full cohort) at the event time for the case in each matched set (i.e. nrisk is the same for all individuals in a matched set).
+#'
+#' @inheritParams smcfcs
+#' @example data-raw/ncc_example.r
+#' @export
+smcfcs.nestedcc <- function(originaldata,smformula,set,event,nrisk,method,predictorMatrix=NULL,m=5,numit=10,rjlimit=1000,noisy=FALSE) {
+  smcfcs.core(originaldata,smtype="nestedcc",smformula,method,predictorMatrix,m,numit,rjlimit,noisy,set=set,event=event,nrisk=nrisk)
+}
+
+#this is the core of the smcfcs function, called by wrapper functions for certain different substantive models
+smcfcs.core <- function(originaldata,smtype,smformula,method,predictorMatrix=NULL,m=5,numit=10,rjlimit=1000,noisy=FALSE,
+                        ...) {
+  #get extra arguments passed in ...
+  extraArgs <- list(...)
 
   stopifnot(is.data.frame(originaldata))
   if (ncol(originaldata)!=length(method)) stop("Method argument must have the same length as the number of columns in the data frame.")
@@ -199,6 +167,9 @@ smcfcs <- function(originaldata,smtype,smformula,method,predictorMatrix=NULL,m=5
   #create matrix of response indicators
   r <- 1*(is.na(originaldata)==0)
 
+  if ((smtype %in% c("lm", "logistic", "poisson", "coxph", "compet", "casecohort","nestedcc"))==FALSE) #RUTH 21/03/2017: ADDED 'nestedcc' OPTION HERE
+    stop(paste("Substantive model type ",smtype," not recognised.",sep=""))
+
   #find column numbers of partially observed, fully observed variables, and outcome
   if (smtype=="coxph") {
 
@@ -211,8 +182,7 @@ smcfcs <- function(originaldata,smtype,smformula,method,predictorMatrix=NULL,m=5
     basehaz <- survival::basehaz(nullMod)
     H0indices <- match(originaldata[,timeCol], basehaz[,2])
     rm(nullMod)
-  }
-  else if (smtype=="compet") {
+  } else if (smtype=="compet") {
 
     timeCol <- (1:dim(originaldata)[2])[colnames(originaldata) %in% toString(as.formula(smformula[[1]])[[2]][[2]])]
     dCol <- (1:dim(originaldata)[2])[colnames(originaldata) %in% toString(as.formula(smformula[[1]])[[2]][[3]][[2]])]
@@ -231,17 +201,66 @@ smcfcs <- function(originaldata,smtype,smformula,method,predictorMatrix=NULL,m=5
       linpred[[cause]] <- as.formula(smformula[[cause]])
     }
     rm(nullMod)
+  } else if (smtype=="casecohort") {
+
+    subcoCol <- (1:dim(originaldata)[2])[colnames(originaldata) %in% extraArgs$in.subco]
+    #subcoMembers is a vector of row numbers of those in the subcohort
+    subcoMembers <- which(originaldata[,subcoCol]==1)
+
+    #generate weights for use in later analysis which we use to obtain baseline cumulative hazard
+    #assign a weight of /samp.frac to individuals in the subcohort and 0 to those outside the subcohort
+    subco.weight<-ifelse(originaldata[,subcoCol]==1,1/extraArgs$sampfrac,0)
+
+
+    entertimeCol <- (1:dim(originaldata)[2])[colnames(originaldata) %in% toString(as.formula(smformula)[[2]][[2]])]
+    timeCol <- (1:dim(originaldata)[2])[colnames(originaldata) %in% toString(as.formula(smformula)[[2]][[3]])]
+    dCol <- (1:dim(originaldata)[2])[colnames(originaldata) %in% toString(as.formula(smformula)[[2]][[4]])]
+    outcomeCol <- c(entertimeCol,timeCol, dCol)
+    d <- originaldata[,dCol]
+
+    #list of unique event times - used in calculation of baseline cumulative hazard
+    list.times=sort(unique(originaldata[,timeCol][originaldata[,dCol]==1]))  #RUTH 21/03/17: ADDED THIS LINE
+
+    smcfcsid <- 1:n
+    smformula2<-paste(smformula,"+cluster(smcfcsid)",sep="")
+  }
+  else if (smtype=="nestedcc") {
+
+    timeCol <- (1:dim(originaldata)[2])[colnames(originaldata) %in% toString(as.formula(smformula)[[2]][[2]])]
+    dCol <- (1:dim(originaldata)[2])[colnames(originaldata) %in% toString(as.formula(smformula)[[2]][[3]])]
+    outcomeCol <- c(timeCol, dCol)
+    setCol <- (1:dim(originaldata)[2])[colnames(originaldata) %in% toString(extraArgs$set)]
+    nriskCol <- (1:dim(originaldata)[2])[colnames(originaldata) %in% toString(extraArgs$nrisk)]
+    eventCol <- (1:dim(originaldata)[2])[colnames(originaldata) %in% toString(extraArgs$event)] #this is distinct from the dCol
+
+    #the below command creates "Surv(t,case)~x" from "Surv(t,case)~x+strata(setno)" (for example) (i.e. it removes the strata part of the formula)
+    #this is used when obtaining outmodxb
+    #note this is done slightly oddly, but this is because as.formula does not work well for long formulas as it splits across lines
+    exp1=as.formula(paste(smformula))[[2]]
+    exp2=as.formula(smformula)[[3]][[2]]
+    smformula2<-paste(deparse(exp1),"~",deparse(exp2,width.cutoff = 500L))
+
+    #This is the indicator of whether an individual ever has the event (regardless of whether they are sometimes used as a control and sometimes (one) as a case)
+    d <- originaldata[,eventCol]
+
+    #noncases is a vector of row numbers of those who never have the event (which is a subset of the controls)
+    noncases <- which(originaldata[,eventCol]==0)
+
+    #number of individuals in each sampled risk set (matched set)
+   num.sampriskset<-ave(rep(1,dim(originaldata)[1]), originaldata[,setCol], FUN = function(x) sum(x))
+
   }
   else {
     outcomeCol <- which(colnames(originaldata)==as.formula(smformula)[[2]])
   }
-
   if (smtype=="compet") {
     smcovnames <- attr(terms(as.formula(smformula[[1]])), "term.labels")
     for (cause in 2:numCauses) {
       smcovnames <- c(smcovnames, attr(terms(as.formula(smformula[[cause]])), "term.labels"))
     }
     smcovnames <- unique(smcovnames)
+  } else  if (smtype=="nestedcc") {
+    smcovnames <- attr(terms(as.formula(smformula)), "term.labels")[-length(attr(terms(as.formula(smformula)), "term.labels"))]
   }
   else {
     smcovnames <- attr(terms(as.formula(smformula)), "term.labels")
@@ -251,19 +270,35 @@ smcfcs <- function(originaldata,smtype,smformula,method,predictorMatrix=NULL,m=5
   #partial vars are those variables for which an imputation method has been specified among the available regression types
   partialVars <- which((method=="norm") | (method=="logreg") | (method=="poisson") | (method=="podds") | (method=="mlogit"))
 
-  if (length(outcomeCol)==1) {
-    if (method[outcomeCol]!="") stop("The element of the method argument corresponding to the outcome variable should be empty.")
-  }
-  else {
-    if (sum(method[outcomeCol]!=c("",""))>0) stop("The elements of the method argument corresponding to the outcome variables should be empty.")
-  }
+  if (length(partialVars)==0) stop("You have not specified any valid imputation methods in the method argument.")
 
-  nonOutcomeCols <- 1:ncol(originaldata)
-  nonOutcomeCols <- nonOutcomeCols[nonOutcomeCols!=outcomeCol]
   #check that methods are given for each partially observed column, and not given for fully observed columns
-  if (all.equal(which(method[nonOutcomeCols]!=""), which(colSums(r[,nonOutcomeCols])!=n), check.names=FALSE)==FALSE)
-    stop("The method argument must have empty \"\" elements corresponding to fully observed columns and non-empty
-         elements for those columns which have missing values.")
+  for (colnum in 1:ncol(originaldata)) {
+    if (method[colnum]!="") {
+      #an imputation method has been specified
+      if (colnum %in% outcomeCol) {
+        stop(paste("An imputation method has been specified for ",colnames(originaldata)[colnum],
+                   ". Elements of the method argument corresponding to the outcome variable(s) should be empty.",sep=""))
+      }
+      else {
+        if (sum(r[,colnum])==n) {
+          stop(paste("An imputation method has been specified for ",colnames(originaldata)[colnum],
+                     ", but it appears to be fully observed.",sep=""))
+        }
+      }
+    }
+    else {
+      #no imputation method has been specified
+      if (sum(r[,colnum])<n) {
+        #some values are missing
+        if ((colnum %in% outcomeCol)==FALSE) {
+          stop(paste("Variable ",colnames(originaldata), " does not have an imputation method specified,
+                     yet appears to have missing values.",sep=""))
+        }
+        }
+      }
+
+    }
 
   #fully observed vars are those that are fully observed and are covariates in the substantive model
   fullObsVars <- which((colSums(r)==n) & (colnames(originaldata) %in% smcovnames))
@@ -291,12 +326,12 @@ smcfcs <- function(originaldata,smtype,smformula,method,predictorMatrix=NULL,m=5
     for (var in 1:length(partialVars)) {
       targetCol <- partialVars[var]
       #if (method[targetCol]=="latnorm") {
-        #initialize latent predictors with single error prone measurement
-        #errorProneCols <- which(errorProneMatrix[targetCol,]==1)
-        #imputations[[imp]][,targetCol] <- apply(imputations[[imp]][,errorProneCols], 1, firstnonna)
+      #initialize latent predictors with single error prone measurement
+      #errorProneCols <- which(errorProneMatrix[targetCol,]==1)
+      #imputations[[imp]][,targetCol] <- apply(imputations[[imp]][,errorProneCols], 1, firstnonna)
       #}
       #else {
-        imputations[[imp]][r[,targetCol]==0,targetCol] <- sample(imputations[[imp]][r[,targetCol]==1,targetCol], size=sum(r[,targetCol]==0), replace=TRUE)
+      imputations[[imp]][r[,targetCol]==0,targetCol] <- sample(imputations[[imp]][r[,targetCol]==1,targetCol], size=sum(r[,targetCol]==0), replace=TRUE)
       #}
     }
 
@@ -358,10 +393,10 @@ smcfcs <- function(originaldata,smtype,smformula,method,predictorMatrix=NULL,m=5
         }
         if ((imp==1) & (cyclenum==1)) {
           #if (method[targetCol]=="latnorm") {
-            #print(paste("Imputing: ",colnames(imputations[[imp]])[targetCol]," using ",paste(colnames(imputations[[imp]])[c(predictorCols,which(errorProneMatrix[targetCol,]==1))],collapse=',')," plus outcome",collapse=','))
+          #print(paste("Imputing: ",colnames(imputations[[imp]])[targetCol]," using ",paste(colnames(imputations[[imp]])[c(predictorCols,which(errorProneMatrix[targetCol,]==1))],collapse=',')," plus outcome",collapse=','))
           #}
           #else {
-            print(paste("Imputing: ",colnames(imputations[[imp]])[targetCol]," using ",paste(colnames(imputations[[imp]])[predictorCols],collapse=',')," plus outcome",collapse=','))
+          print(paste("Imputing: ",colnames(imputations[[imp]])[targetCol]," using ",paste(colnames(imputations[[imp]])[predictorCols],collapse=',')," plus outcome",collapse=','))
           #}
         }
         if (length(predictorCols)>0) {
@@ -370,9 +405,16 @@ smcfcs <- function(originaldata,smtype,smformula,method,predictorMatrix=NULL,m=5
         else {
           xmodformula <- as.formula(paste(colnames(imputations[[imp]])[targetCol], "~1",sep=""))
         }
+        if (smtype=="casecohort") {
+          xmoddata <- imputations[[imp]][subcoMembers,]
+        } else if (smtype=="nestedcc"){
+          xmoddata <- imputations[[imp]][noncases,]
+        } else {
+          xmoddata <- imputations[[imp]]
+        }
         if ((method[targetCol]=="norm") | (method[targetCol]=="latnorm")) {
           #estimate parameters of covariate model
-          xmod <- lm(xmodformula, data=imputations[[imp]])
+          xmod <- lm(xmodformula, data=xmoddata)
           #take draw from posterior of covariate model parameters
           beta <- xmod$coef
           sigmasq <- summary(xmod)$sigma^2
@@ -380,31 +422,41 @@ smcfcs <- function(originaldata,smtype,smformula,method,predictorMatrix=NULL,m=5
           covariance <- (newsigmasq/sigmasq)*vcov(xmod)
           newbeta = beta + MASS::mvrnorm(1, mu=rep(0,ncol(covariance)), Sigma=covariance)
           #calculate fitted values
-          xfitted <- model.matrix(xmod) %*% newbeta
-        }
-        else if (method[targetCol]=="logreg") {
-          xmod <- glm(xmodformula, family="binomial",data=imputations[[imp]])
+          if ((smtype=="casecohort")|(smtype=="nestedcc")) {
+            xfitted <- model.matrix(xmodformula, data=imputations[[imp]]) %*% newbeta
+          } else {
+            xfitted <- model.matrix(xmod) %*% newbeta
+          }
+        } else if (method[targetCol]=="logreg") {
+          xmod <- glm(xmodformula, family="binomial",data=xmoddata)
           newbeta = modPostDraw(xmod)
-          xfitted <- expit(model.matrix(xmod) %*% newbeta)
-        }
-        else if (method[targetCol]=="poisson") {
-          xmod <- glm(xmodformula, family="poisson", data=imputations[[imp]])
+          if ((smtype=="casecohort")|(smtype=="nestedcc")) {
+            xfitted <- expit(model.matrix(xmodformula, data=imputations[[imp]]) %*% newbeta)
+          } else {
+            xfitted <- expit(model.matrix(xmod) %*% newbeta)
+          }
+        } else if (method[targetCol]=="poisson") {
+          xmod <- glm(xmodformula, family="poisson", data=xmoddata)
           newbeta = modPostDraw(xmod)
-          xfitted <- exp(model.matrix(xmod) %*% newbeta)
-        }
-        else if (method[targetCol]=="podds") {
+          if ((smtype=="casecohort")|(smtype=="nestedcc")) {
+            xfitted <- exp(model.matrix(xmodformula, data=imputations[[imp]]) %*% newbeta)
+          } else {
+            xfitted <- exp(model.matrix(xmod) %*% newbeta)
+          }
+        } else if (method[targetCol]=="podds") {
           if (is.ordered(imputations[[imp]][,targetCol])==FALSE) stop("Variables to be imputed using method podds must be stored as ordered factors.")
-          xmod <- VGAM::vglm(xmodformula, VGAM::propodds, data=imputations[[imp]])
+          xmod <- VGAM::vglm(xmodformula, VGAM::propodds, data=xmoddata)
+          xmod.dummy <- VGAM::vglm(xmodformula, VGAM::propodds, data=imputations[[imp]])
           newbeta <- VGAM::coef(xmod) + MASS::mvrnorm(1, mu=rep(0,ncol(VGAM::vcov(xmod))), Sigma=VGAM::vcov(xmod))
-          linpreds <- matrix((VGAM::model.matrix(xmod)) %*% newbeta, byrow=TRUE, ncol=(nlevels(imputations[[imp]][,targetCol])-1))
+          linpreds <- matrix((VGAM::model.matrix(xmod.dummy)) %*% newbeta, byrow=TRUE, ncol=(nlevels(imputations[[imp]][,targetCol])-1))
           cumprobs <- cbind(1/(1+exp(linpreds)), rep(1,nrow(linpreds)))
           xfitted <- cbind(cumprobs[,1] ,cumprobs[,2:ncol(cumprobs)] - cumprobs[,1:(ncol(cumprobs)-1)])
-        }
-        else if (method[targetCol]=="mlogit") {
+        } else if (method[targetCol]=="mlogit") {
           if (is.factor(imputations[[imp]][,targetCol])==FALSE) stop("Variables to be imputed using method modds must be stored as factors.")
-          xmod <- VGAM::vglm(xmodformula, VGAM::multinomial(refLevel=1), data=imputations[[imp]])
+          xmod <- VGAM::vglm(xmodformula, VGAM::multinomial(refLevel=1), data=xmoddata)
+          xmod.dummy <- VGAM::vglm(xmodformula, VGAM::multinomial(refLevel=1), data=imputations[[imp]])
           newbeta <- VGAM::coef(xmod) + MASS::mvrnorm(1, mu=rep(0,ncol(VGAM::vcov(xmod))), Sigma=VGAM::vcov(xmod))
-          linpreds <- matrix((VGAM::model.matrix(xmod)) %*% newbeta, byrow=TRUE, ncol=(nlevels(imputations[[imp]][,targetCol])-1))
+          linpreds <- matrix((VGAM::model.matrix(xmod.dummy)) %*% newbeta, byrow=TRUE, ncol=(nlevels(imputations[[imp]][,targetCol])-1))
           denom <- 1+rowSums(exp(linpreds))
           xfitted <-cbind(1/denom, exp(linpreds) / denom)
         }
@@ -413,29 +465,29 @@ smcfcs <- function(originaldata,smtype,smformula,method,predictorMatrix=NULL,m=5
         }
 
         #if latent normal, estimate error variance and calculate required conditional expectation and variance
-#         if (method[targetCol]=="latnorm") {
-#           errorProneCols <- which(errorProneMatrix[targetCol,]==1)
-#           wmean <- rowMeans(imputations[[imp]][,errorProneCols], na.rm=TRUE)
-#           n_i <- apply(imputations[[imp]][,errorProneCols], 1, sumna)
-#           sum_ni <- sum(n_i)
-#           #estimate error variance
-#           if (cyclenum==1) {
-#             xmat <- matrix(wmean, nrow=nrow(imputations[[imp]]), ncol=length(errorProneCols))
-#             uVec <- c(as.matrix(imputations[[imp]][,errorProneCols] - xmat))
-#             sigmausq <- sum(uVec^2, na.rm=TRUE) / (sum_ni - n)
-#           }
-#           else {
-#             xmat <- matrix(imputations[[imp]][,targetCol], nrow=nrow(imputations[[imp]]), ncol=length(errorProneCols))
-#             uVec <- c(as.matrix(imputations[[imp]][,errorProneCols] - xmat))
-#             sigmausq <- sum(uVec^2, na.rm=TRUE) / sum_ni
-#           }
-#           #take draw from posterior of error variance
-#           sigmausq <- sigmausq*sum_ni/rchisq(1,sum_ni)
-#           #calculate conditional mean and variance
-#           lambda <- newsigmasq/(newsigmasq+sigmausq/n_i)
-#           xfitted <- xfitted + lambda * (wmean - xfitted)
-#           newsigmasq <- newsigmasq*(1-lambda)
-#         }
+        #         if (method[targetCol]=="latnorm") {
+        #           errorProneCols <- which(errorProneMatrix[targetCol,]==1)
+        #           wmean <- rowMeans(imputations[[imp]][,errorProneCols], na.rm=TRUE)
+        #           n_i <- apply(imputations[[imp]][,errorProneCols], 1, sumna)
+        #           sum_ni <- sum(n_i)
+        #           #estimate error variance
+        #           if (cyclenum==1) {
+        #             xmat <- matrix(wmean, nrow=nrow(imputations[[imp]]), ncol=length(errorProneCols))
+        #             uVec <- c(as.matrix(imputations[[imp]][,errorProneCols] - xmat))
+        #             sigmausq <- sum(uVec^2, na.rm=TRUE) / (sum_ni - n)
+        #           }
+        #           else {
+        #             xmat <- matrix(imputations[[imp]][,targetCol], nrow=nrow(imputations[[imp]]), ncol=length(errorProneCols))
+        #             uVec <- c(as.matrix(imputations[[imp]][,errorProneCols] - xmat))
+        #             sigmausq <- sum(uVec^2, na.rm=TRUE) / sum_ni
+        #           }
+        #           #take draw from posterior of error variance
+        #           sigmausq <- sigmausq*sum_ni/rchisq(1,sum_ni)
+        #           #calculate conditional mean and variance
+        #           lambda <- newsigmasq/(newsigmasq+sigmausq/n_i)
+        #           xfitted <- xfitted + lambda * (wmean - xfitted)
+        #           newsigmasq <- newsigmasq*(1-lambda)
+        #         }
 
         #estimate parameters of substantive model
         if (smtype=="lm") {
@@ -486,6 +538,34 @@ smcfcs <- function(originaldata,smtype,smformula,method,predictorMatrix=NULL,m=5
             }
           }
         }
+        else if (smtype=="casecohort") {
+          ymod <- survival::coxph(as.formula(smformula2), imputations[[imp]])
+          outcomeModBeta <- modPostDraw(ymod)
+
+          cumhaz.denom.elements=exp(model.matrix(as.formula(smformula),imputations[[imp]])[,-1] %*% outcomeModBeta)
+          cumhaz.denom=sapply(list.times,function(x){sum(cumhaz.denom.elements[which(originaldata[,timeCol]>=x)]*subco.weight[which(originaldata[,timeCol]>=x)])})
+          exp.func.denom=cumsum(1/cumhaz.denom)
+          H0.fun=stepfun(list.times,c(0,exp.func.denom))
+          H0=H0.fun(originaldata[,timeCol])
+
+          if (noisy==TRUE) {
+            print(summary(ymod))
+          }
+        }
+        else if (smtype=="nestedcc") {
+          ymod <- survival::coxph(as.formula(smformula), imputations[[imp]])
+          outcomeModBeta <- modPostDraw(ymod)
+
+          explan.matrix<-model.matrix(ymod)
+          cumbasehaz.denom<-exp(matrix(outcomeModBeta,nrow=1)%*%t(explan.matrix))*originaldata[,nriskCol]/num.sampriskset
+          cumbasehaz.denom<-ave(cumbasehaz.denom, originaldata[,setCol], FUN = sum)[originaldata[,dCol]==1] #this is the denominator of the contribution to the cumulative baseline hazard at each event time
+          cumbasehaz.t<-originaldata[,timeCol][originaldata[,dCol]==1] #times to which the baseline cumulative hazards refer
+          H0<-unlist(lapply(originaldata[,timeCol],function(x) {sum((1/cumbasehaz.denom)[cumbasehaz.t<=x])}))
+
+          if (noisy==TRUE) {
+            print(summary(ymod))
+          }
+        }
 
         if ((imp==1) & (cyclenum==1) & (var==1)) {
           if (smtype=="compet") {
@@ -561,16 +641,21 @@ smcfcs <- function(originaldata,smtype,smformula,method,predictorMatrix=NULL,m=5
               outmodxb <-  model.matrix(as.formula(smformula),imputations[[imp]]) %*% outcomeModBeta
               outcomeDens <- dpois(imputations[[imp]][imputationNeeded,outcomeCol], exp(outmodxb[imputationNeeded]))
             }
-            else if (smtype=="coxph") {
+            else if ((smtype=="coxph") | (smtype=="casecohort")) {
               outmodxb <-  model.matrix(as.formula(smformula),imputations[[imp]])
-              outmodxb <- outmodxb[,2:dim(outmodxb)[2]] %*% outcomeModBeta
+              outmodxb <- as.matrix(outmodxb[,2:dim(outmodxb)[2]]) %*% as.matrix(outcomeModBeta)
+              outcomeDens <- exp(-H0[imputationNeeded] * exp(outmodxb[imputationNeeded]))* (exp(outmodxb[imputationNeeded])^d[imputationNeeded])
+            }
+            else if (smtype=="nestedcc") {
+              outmodxb <-  model.matrix(as.formula(smformula2),imputations[[imp]])
+              outmodxb <- as.matrix(outmodxb[,2:dim(outmodxb)[2]]) %*% as.matrix(outcomeModBeta)
               outcomeDens <- exp(-H0[imputationNeeded] * exp(outmodxb[imputationNeeded]))* (exp(outmodxb[imputationNeeded])^d[imputationNeeded])
             }
             else if (smtype=="compet") {
               outcomeDens <- rep(1,length(imputationNeeded))
               for (cause in 1:numCauses) {
                 outmodxb <-  model.matrix(linpred[[cause]],imputations[[imp]])
-                outmodxb <- outmodxb[,2:dim(outmodxb)[2]] %*% outcomeModBeta[[cause]]
+                outmodxb <- as.matrix(outmodxb[,2:dim(outmodxb)[2]]) %*% as.matrix(outcomeModBeta[[cause]])
                 outcomeDens <- outcomeDens * exp(-H0[[cause]][imputationNeeded] * exp(outmodxb[imputationNeeded]))* (exp(outmodxb[imputationNeeded])^(d[imputationNeeded]==cause))
               }
             }
@@ -631,9 +716,17 @@ smcfcs <- function(originaldata,smtype,smformula,method,predictorMatrix=NULL,m=5
               prob = dpois(imputations[[imp]][imputationNeeded,outcomeCol], exp(outmodxb[imputationNeeded]))
               reject = 1*(uDraw>prob)
             }
-            else if (smtype=="coxph") {
+            else if ((smtype=="coxph") | (smtype=="casecohort")) {
               outmodxb <-  model.matrix(as.formula(smformula),imputations[[imp]])
-              outmodxb <- outmodxb[,2:dim(outmodxb)[2]] %*% outcomeModBeta
+              outmodxb <- as.matrix(outmodxb[,2:dim(outmodxb)[2]]) %*% as.matrix(outcomeModBeta)
+              s_t = exp(-H0[imputationNeeded]* exp(outmodxb[imputationNeeded]))
+              prob = exp(1 + outmodxb[imputationNeeded] - (H0[imputationNeeded]* exp(outmodxb[imputationNeeded])) ) * H0[imputationNeeded]
+              prob = d[imputationNeeded]*prob + (1-d[imputationNeeded])*s_t
+              reject = 1*(uDraw > prob )
+            }
+            else if (smtype=="nestedcc") {
+              outmodxb <-  model.matrix(as.formula(smformula2),imputations[[imp]])
+              outmodxb <- as.matrix(outmodxb[,2:dim(outmodxb)[2]]) %*% as.matrix(outcomeModBeta)
               s_t = exp(-H0[imputationNeeded]* exp(outmodxb[imputationNeeded]))
               prob = exp(1 + outmodxb[imputationNeeded] - (H0[imputationNeeded]* exp(outmodxb[imputationNeeded])) ) * H0[imputationNeeded]
               prob = d[imputationNeeded]*prob + (1-d[imputationNeeded])*s_t
@@ -643,7 +736,7 @@ smcfcs <- function(originaldata,smtype,smformula,method,predictorMatrix=NULL,m=5
               prob <- rep(1,length(imputationNeeded))
               for (cause in 1:numCauses) {
                 outmodxb <-  model.matrix(linpred[[cause]],imputations[[imp]])
-                outmodxb <- outmodxb[,2:dim(outmodxb)[2]] %*% outcomeModBeta[[cause]]
+                outmodxb <- as.matrix(outmodxb[,2:dim(outmodxb)[2]]) %*% as.matrix(outcomeModBeta[[cause]])
                 prob = prob * exp(-H0[[cause]][imputationNeeded] * exp(outmodxb[imputationNeeded]))* (H0[[cause]][imputationNeeded]*exp(1+outmodxb[imputationNeeded]))^(d[imputationNeeded]==cause)
               }
               reject = 1*(uDraw > prob )
@@ -692,9 +785,17 @@ smcfcs <- function(originaldata,smtype,smformula,method,predictorMatrix=NULL,m=5
               prob = dpois(tempData[,outcomeCol], exp(outmodxb))
               reject = 1*(uDraw>prob)
             }
-            else if (smtype=="coxph") {
+            else if ((smtype=="coxph") | (smtype=="casecohort")) {
               outmodxb <-  model.matrix(as.formula(smformula),tempData)
-              outmodxb <- outmodxb[,2:dim(outmodxb)[2]] %*% outcomeModBeta
+              outmodxb <- as.matrix(outmodxb[,2:dim(outmodxb)[2]]) %*% as.matrix(outcomeModBeta)
+              s_t = exp(-H0[i]* exp(outmodxb))
+              prob = exp(1 + outmodxb - (H0[i]* exp(outmodxb)) ) * H0[i]
+              prob = d[i]*prob + (1-d[i])*s_t
+              reject = 1*(uDraw > prob )
+            }
+            else if (smtype=="nestedcc") {
+              outmodxb <-  model.matrix(as.formula(smformula2),tempData)
+              outmodxb <- as.matrix(outmodxb[,2:dim(outmodxb)[2]]) %*% as.matrix(outcomeModBeta)
               s_t = exp(-H0[i]* exp(outmodxb))
               prob = exp(1 + outmodxb - (H0[i]* exp(outmodxb)) ) * H0[i]
               prob = d[i]*prob + (1-d[i])*s_t
@@ -704,7 +805,7 @@ smcfcs <- function(originaldata,smtype,smformula,method,predictorMatrix=NULL,m=5
               prob <- rep(1,rjlimit)
               for (cause in 1:numCauses) {
                 outmodxb <-  model.matrix(linpred[[cause]],tempData)
-                outmodxb <- outmodxb[,2:dim(outmodxb)[2]] %*% outcomeModBeta[[cause]]
+                outmodxb <- as.matrix(outmodxb[,2:dim(outmodxb)[2]]) %*% as.matrix(outcomeModBeta[[cause]])
                 prob = prob * exp(-H0[[cause]][i] * exp(outmodxb))* (H0[[cause]][i]*exp(1+outmodxb))^(d[i]==cause)
               }
               reject = 1*(uDraw > prob )
